local MAJOR = "LibScriptableDisplayWidgetTimer-1.0" 
local MINOR = 14

assert(LibStub, MAJOR.." requires LibStub") 
local WidgetTimer = LibStub:NewLibrary(MAJOR, MINOR)
if not WidgetTimer then return end
local LibWidget = LibStub("LibScriptableDisplayWidget-1.0", true)
assert(LibWidget, MAJOR .. " requires LibScriptableDisplayWidget-1.0")
local LibTimer = LibStub("LibScriptableDisplayTimer-1.0", true)
assert(LibTimer, MAJOR .. " requires LibScriptableDisplayTimer-1.0")
local Evaluator = LibStub("LibScriptableDisplayEvaluator-1.0", true)
assert(Evaluator, MAJOR .. " requires LibScriptableDisplayEvaluator-1.0")

local pool = setmetatable({}, {__mode = "k"})

if not WidgetTimer.__index then WidgetTimer.__index = WidgetTimer end

WidgetTimer.defaults = {
	expression = 'return',
	repeating = false,
	update = 0,
}

local widgetType = {timer=true}

-- Create a new LibScriptableDisplayWidgetTimer object
-- @name WidgetTimer.New
-- @usage WidgetTimer:New(visitor, name, config, errorLevel)
-- @param visitor An LibScriptableDisplayCore-1.0 object, or provide your own
-- @param name A name for the timer widget
-- @param config This timeer's parameters
-- @param errorLevel The errorLevel for this object
-- @param An optional timer. This should have a :Start() and :Stop().
-- @return A new LibScriptableDisplayWidgetTimer widget
function WidgetTimer:New(visitor, name, config, errorLevel, timer) 
	assert(name, "WidgetTimer requires a name.")
	assert(config, "Please provide the timer with a config")
	assert(config.expression, name .. ": Please provide the timer with an expression")
	
	local obj = next(pool)

	if obj then
		pool[obj] = nil
	else
		obj = {}
		obj.options = {}
	end
		
	setmetatable(obj, self)

	obj.widget = LibWidget:New(obj, visitor, name, config, 0, 0, 0, widgetType, errorLevel)
	
	obj.expression = config.expression
	obj.repeating = config.repeating
	obj.update = config.update or WidgetTimer.defaults.update
	obj.disabled = config.disabled
		
	obj.timer = timer or LibTimer:New("WidgetTimer.timer " .. obj.name, obj.update, obj.repeating, self.Update, obj, obj.errorLevel)
	
	return obj	
end


-- Delete a LibScriptableDisplayWidgetTimer object
-- @name LibScriptableDisplayWidgetTimer.Del
-- @usage LibScriptableDisplayWidgetTimer:Del([marq])
-- @param marq A LibScriptableDisplayWidgetTimer object
-- @return Nothing
function WidgetTimer:Del(timer)
	if not timer then
		timer = self
	end
	timer:Stop()
	timer.widget:Del()
	timer.timer:Del()
	pool[timer] = true
end

-- Start a LibScriptableDisplayWidgetTimer
-- @name LibScriptableDisplayWidgetTimer.Start
-- @usage object:Start()
-- @return Nothing
function WidgetTimer:Start()
	self.error:Print("WidgetTimer:Start - " .. self.name, 2)
	if self.update > 0 and not self.disabled and not self.active then
		self.timer:Start()
		self.active = true
	end
end

-- Stop a LibScriptableDisplayWidgetTimer
-- @name LibScriptableDisplayWidgetTimer.Stop
-- @usage object:Stop()
-- @return Nothing
function WidgetTimer:Stop()
	self.error:Print("WidgetTimer:Stop - " .. self.name, 2)
	self.timer:Stop()
	self.active = false
end

-- Update data
-- @name LibScriptableDisplayWidgetTimer.Update
-- @name LibScriptableDisplayWidgetTimer.Update
-- @usage LibScriptableDisplayWidgetTimer:Update()
-- @return Nothing
function WidgetTimer:Update()
	self.error:Print("WidgetTimer:Update - " .. self.name)
	self.visitor.environment.self = self
	Evaluator.ExecuteCode(self.visitor.environment, self.name, self.expression)
	self.visitor.environment.self = nil
end

function WidgetTimer:GetOptions(visitor, db)
		local defaults = WidgetTimer.defaults
		local options = {
			enabled = {
				name = "Enabled",
				desc = "Whether this timer is enabled or not",
				type = "toggle",
				get = function() return db.enabled end,
				set = function(info, v) 
					db.enabled = v
					if visitor.RestartTimers then
						visitor:RestartTimers()
					end					
					db["enabledDirty"] = true
				end,
				order = 2
			},
			update = {
				name = "Update Rate",
				desc = "Enter the timer's refresh rate",
				type = "input",
				pattern = "%d",
				get = function()
					return tostring(db.update or defaults.update)
				end,
				set = function(info, v)
					db.update = tonumber(v)
					if visitor.RestartTimers then
						visitor:RestartTimers()
					end
					db["updateDirty"] = true
				end,
				order = 3
			},
			repeating = {
				name = "Repeating",
				desc = "Toggle whether to repeat this timer",
				type = "toggle",
				get = function()
					return db.repeating or defaults.repeating
				end,
				set = function(info, v)
					db.repeating = v
					if visitor.RestartTimers then
						visitor:RestartTimers()
					end
					db["repeatingDirty"] = true
				end,
				order = 4
			},
			expression = {
				name = "Expression",
				desc = "Enter this widget's expression",
				type = "input",
				width = "full",
				multiline = true,
				get = function()
					return db.expression or defaults.expression
				end,
				set = function(info, v)
					db.expression = v
					if visitor.RestartTimers then
						visitor:RestartTimers()
					end
					db["expressionDirty"] = true
				end,
				order = 5
			},
			
		}
	return options
end
